#include "stdlib.h"
#include "intarray.h"

/** implementation of the data structure **/

struct _intarray {
  size_t ia_size; // Current number of elements.
  int *ia_mem;	  // Actual array.
  int ia_def;	// Default value.
};

intarray *ia_new(size_t initial_size, int default_value) {
  intarray *r = malloc (sizeof (struct _intarray));
  if (r != NULL) {
    r->ia_size = initial_size;
    r->ia_def = default_value;
    r->ia_mem = malloc (initial_size * sizeof (int));
    if (r->ia_mem != NULL) { // initialize the array to default_value
      for (int i=0; i<initial_size; i++) {
	*(r->ia_mem + i) = default_value;
      }
    } else { // internal malloc failed
      free (r);
      r = NULL;
    }
  }
  return r;
}

void ia_destroy (intarray *ia) {
  if (ia != NULL) {
    free (ia->ia_mem); 		/* free interal array */
    free (ia);			/* free the struct */
  }
}

int ia_read(intarray * ia, size_t i) {
  if (ia == NULL) {
    return -1;
  } else { // ia != NULL
    if (i < ia->ia_size) { // index i is valid!
      return *(ia->ia_mem + i);
    } else { 			/* index is bigger than array */
      return ia->ia_def;
    }
  }
}

int ia_write (intarray *ia, size_t i, int val) {
  if (ia == NULL) {
    return -1;
  } else {
    if (i < ia->ia_size) { 	/* index is valid */
      *(ia->ia_mem + i) = val;
      return 0;
    } else { 			/* index is bigger */
      int newsize = i+1;	/* so that i is a valid index */
      ia->ia_mem = realloc (ia->ia_mem, newsize * sizeof (int));
      if (ia->ia_mem == NULL) {
	return -1;
      }
      // initialize the new part of the array
      for (int j=ia->ia_size; j<i; j++){
	*(ia->ia_mem + j) = ia->ia_def;
      }
      *(ia->ia_mem + i) = val;
      ia->ia_size = newsize;
      return newsize;
    }
  }
}
